#include "HX711.h"
#include "soc/rtc.h"
#include <Adafruit_SSD1306.h>
#include <Keypad.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

HX711 scale;
String weightNumber;

float load;
String message;
String countMessage;
//int LED_BUILTIN = 2;

int countTimesFilled = 0;

//KEYPAD setup
const byte ROWS = 4; //four rows
const byte COLS = 3; //three columns
char keys[ROWS][COLS] = {
    {'1', '2', '3'},
    {'4', '5', '6'},
    {'7', '8', '9'},
    {'*', '0', '#'}};

byte rowPins[ROWS] = {27, 26, 25, 33}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {13, 12, 14};     //connect to the column pinouts of the keypad

Keypad keypad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

String inputString;
long inputInt;

void displayString(String dispString, int x, int y, int size)
{
  display.clearDisplay();
  display.setTextSize(size);
  display.setTextColor(WHITE);
  display.setCursor(x, y);
  display.println(dispString);
  display.display();
}

void displayChar(String dispChar, int x, int y, int size)
{
  display.clearDisplay();
  display.setTextSize(size);
  display.setTextColor(WHITE);
  display.setCursor(x, y);
  display.println(dispChar);
  display.display();
}

void resetFilling()
{
  Serial.println("Times counted_1: ");
  Serial.print(countTimesFilled);
  countTimesFilled += 1;
  Serial.println("Times counted_2: ");
  Serial.print(countTimesFilled);
  while (true)
  {
    if (keypad.getKey() == '*')
    {
      break;
    }
  }
}

void cupWeight(float limit)
{
  boolean escape = false;
  while (true)
  {
    char keyClicked = keypad.getKey();
    load = scale.get_units();
    message = String(load, 2);
    // displayString(message, 1, 1, 4);
    if (keyClicked == '#')
    {
      displayString("RESET     * -> YES  # -> NO", 0, 0, 2);

      while (true)
      {
        keyClicked = keypad.getKey();
        if (keyClicked == '*')
        {
          countTimesFilled = 0;
          escape = true;
          break;
        }
        if (keyClicked == '#')
        {
          break;
        }
      }
      if (escape)
      {
        displayString("RESET", 0, 0, 4);
        delay(1000);
        displayString("Enter  weight", 0, 0, 3);
        break;
      }
    }
    else if (keyClicked == '*')
    {
      countMessage = "Cups      filled:";
      countMessage += countTimesFilled;
      
      displayString(countMessage, 0, 0, 2);
      delay(4000);
    }
    else if (load >= limit)
    {
      digitalWrite(LED_BUILTIN, HIGH);
      displayString("Remove cup", 1, 1, 3);
      delay(80);
      // countTimesFilled += 1;
      resetFilling();
      digitalWrite(LED_BUILTIN, LOW);
    }
    else if (load <= 1)
    {
      displayString("0", 1, 1, 3);
    }
    else
    {
      displayString(message, 1, 1, 3);
      // digitalWrite(LED_BUILTIN, LOW);
    }
  }
}

void setup()
{
  Serial.begin(115200);
  //rtc_clk_cpu_freq_set(RTC_CPU_FREQ_80M);
  if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))
  { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for (;;)
      ;
  }

  pinMode(LED_BUILTIN, OUTPUT);

  inputString.reserve(4);

  delay(2000);
  display.clearDisplay();

  Serial.println("Initializing the scale");

  scale.begin(4, 5);

  scale.set_scale(1058.09); // this value is obtained by calibrating the scale with known weights; see the README for details
  scale.tare();             // reset the scale to 0

  displayString("Ready", 0, 0, 3);
  delay(1000);
  displayString("Enter  weight", 0, 0, 3);

  weightNumber = "";
}

void loop()
{

  char key = keypad.getKey();

  // displayString("Enter weight", 0, 0, 3);

  if (key >= '0' && key <= '9')
  {                     // only act on numeric keys
    inputString += key; // append new character to input string
    displayString(inputString, 0, 0, 3);
    // digitalWrite(LED_BUILTIN, HIGH);
  }
  else if (key == '#')
  {
    if (inputString.length() > 0)
    {
      inputInt = inputString.toInt(); // YOU GOT AN INTEGER NUMBER
      float inputFloat = inputString.toFloat();
      displayString("Saved: " + inputString, 0, 0, 3);
      delay(2000);
      inputString = ""; // clear input

      displayString("START     PUMPING", 0, 0, 2);
      delay(600);
      // DO YOUR WORK HERE

      cupWeight(inputFloat);
    }
  }
  else if (key == '*')
  {
    inputString = ""; // clear input
    displayString(inputString, 0, 0, 3);
  }
}
